#ifndef _OPENDRIVE_LINK_H
#define _OPENDRIVE_LINK_H

#include "opendrive_common.h"
#include "g2_spline.h"
#include "opendrive_line.h"
#include "opendrive_spiral.h"
#include "opendrive_arc.h"
#include "opendrive_param_poly3.h"
#include "opendrive_road_node.h"

class COpendriveLink
{
  friend class COpendriveRoadSegment;
  friend class COpendriveRoadNode;
  friend class COpendriveRoad;
  private:
    COpendriveRoadNode *prev;
    COpendriveRoadNode *next;
    COpendriveRoadSegment *segment;
    COpendriveLane *lane;
    CG2Spline *spline;
    opendrive_mark_t mark;
    double resolution;
    double scale_factor;    
  protected:
    COpendriveLink();
    COpendriveLink(const COpendriveLink &object);
    void set_prev(COpendriveRoadNode *node);
    void set_next(COpendriveRoadNode *node);
    void set_road_mark(opendrive_mark_t mark);
    void set_parent_segment(COpendriveRoadSegment *segment);
    void set_parent_lane(COpendriveLane *lane);
    void set_resolution(double res);
    void set_scale_factor(double scale);
    void generate(double start_curvature,double end_curvature);
    void update_references(segment_up_ref_t &segment_refs,lane_up_ref_t &lanei_refs,node_up_ref_t &node_refs);
    bool clean_references(node_up_ref_t &refs);
    void update_start_pose(TOpendriveWorldPose &start,double curvature);
    void update_end_pose(TOpendriveWorldPose &end,double curvature);
  public:
    const COpendriveRoadNode &get_prev(void) const;
    const COpendriveRoadNode &get_next(void) const;
    opendrive_mark_t get_road_mark(void) const;
    const COpendriveRoadSegment &get_parent_segment(void) const;
    const COpendriveLane &get_parent_lane(void) const;
    double get_resolution(void) const;
    double get_scale_factor(void) const;
    double find_closest_world_pose(TOpendriveWorldPose &world,TOpendriveWorldPose &pose);
    void get_trajectory(std::vector<double> &x, std::vector<double> &y,double start_length=0.0, double end_length=-1.0) const;
    void get_trajectory(std::vector<double> &x, std::vector<double> &y, std::vector<double> &yaw,double start_length=0.0, double end_length=-1.0) const;
    double get_length(void) const;
    TOpendriveWorldPose get_pose_at(double length);
    double get_curvature_at(double length);
    friend std::ostream& operator<<(std::ostream& out, COpendriveLink &link);
    ~COpendriveLink();
};

#endif
