#ifndef _OPENDRIVE_SIGNAL_H
#define _OPENDRIVE_SIGNAL_H

#include "opendrive_common.h"
#include "opendrive_road_segment.h"
#ifdef _HAVE_XSD
#include "xml/OpenDRIVE_1.4H.hxx"
#endif

class COpendriveRoadSegment;

class COpendriveSignal
{
  friend class COpendriveRoadSegment;
  private:
    COpendriveRoadSegment *segment;
    int id; ///< Signal's id.
    TOpendriveTrackPose pose;
    std::string type; ///< Signal's OpenDrive type.
    std::string sub_type; ///< Signal's OpenDrive subtype.
    int value; ///< Signal's value.
    std::string text; ///< Signal's text.    
    double scale_factor;
  protected:
    void load(signals::signal_type &signal_info,COpendriveRoadSegment *segment);
    void update_references(segment_up_ref_t &segment_refs);
    void set_scale_factor(double scale);
  public:
    COpendriveSignal();
    COpendriveSignal(const COpendriveSignal &object);
    double get_scale_factor(void);
    int get_id(void) const;
    TOpendriveTrackPose get_track_pose(void) const;
    TOpendriveWorldPose get_world_pose(void) const;
    void get_type(std::string &type, std::string &sub_type) const;
    int get_value(void) const;
    std::string get_text(void) const;
    friend std::ostream& operator<<(std::ostream& out, COpendriveSignal &signal);
    ~COpendriveSignal();
};

#endif
