#include "opendrive_arc.h"
#include <cmath>

COpendriveArc::COpendriveArc()
{
  this->curvature=0.0;
}

COpendriveArc::COpendriveArc(const COpendriveArc &object) : COpendriveGeometry(object)
{
  this->curvature=object.curvature;
}

bool COpendriveArc::transform_local_pose(const TOpendriveTrackPose &track,TOpendriveLocalPose &local) const
{
  double alpha;

  alpha = track.s*this->curvature*this->scale_factor;
  local.u = std::sin(alpha)/(this->curvature*this->scale_factor) - track.t*std::sin(alpha);
  local.v = (1 - std::cos(alpha))/(this->curvature*this->scale_factor) + track.t*std::cos(alpha);
  local.heading = normalize_angle(track.heading + alpha);
  return true;
}

void COpendriveArc::print(std::ostream &out)
{
  double start_curvature,end_curvature;

  COpendriveGeometry::print(out);
  this->get_curvature(start_curvature,end_curvature);
  out << "          curvature = " << start_curvature << std::endl;
}

std::string COpendriveArc::get_name(void)
{
  return std::string("arc");
}

void COpendriveArc::load_params(const planView::geometry_type &geometry_info)
{
  this->curvature = (geometry_info.arc().get().curvature().present() ? geometry_info.arc().get().curvature().get() : 0.0);
}

COpendriveGeometry *COpendriveArc::clone(void)
{
  COpendriveArc *new_arc=new COpendriveArc(*this);

  return new_arc;
}

void COpendriveArc::get_curvature(double &start,double &end)
{
  start=this->curvature*this->scale_factor;
  end=this->curvature*this->scale_factor;
}

void COpendriveArc::operator=(const COpendriveArc &object)
{
  COpendriveGeometry::operator=(object);
  this->curvature=object.curvature;
}

COpendriveArc::~COpendriveArc()
{
  this->curvature=0.0;
}

